@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 """
-PCILeech Firmware Generator - Unified Entry Point with Requirements Enforcement
+PCILeech Firmware Generator - Unified Entry Point
 
 This is the single entry point for all PCILeech functionality with automatic
 dependency checking and installation.
@@ -20,6 +20,19 @@ sys.path.insert(0, str(project_root))
 sys.path.insert(0, str(project_root / "src"))
 
 
+def get_version():
+    """Get the current version from the centralized version resolver."""
+    try:
+        from src.utils.version_resolver import get_version_info
+
+        version_info = get_version_info()
+        title = version_info.get("title", "PCILeech Firmware Generator")
+        version = version_info.get("version", "unknown")
+        return f"{title} v{version}"
+    except ImportError:
+        return "PCILeech Firmware Generator (version unknown)"
+
+
 class RequirementsError(Exception):
     """Raised when requirements cannot be satisfied."""
 
@@ -105,6 +118,9 @@ def check_and_install_requirements():
 
 def is_package_available(package_name):
     """Check if a package is available for import."""
+    if not package_name:
+        return False
+
     # Handle package name mappings (PyPI name vs import name)
     import_mappings = {
         "pyyaml": "yaml",
@@ -122,6 +138,8 @@ def is_package_available(package_name):
     }
 
     import_name = import_mappings.get(package_name.lower(), package_name)
+    if not import_name:
+        return False
 
     try:
         importlib.import_module(import_name)
@@ -258,6 +276,7 @@ try:
         log_warning_safe,
         safe_format,
     )
+    from src.utils.validation_constants import KNOWN_DEVICE_TYPES
 except ImportError as e:
     print(f"❌ Failed to import PCILeech modules: {e}")
     print("Make sure you're running from the PCILeech project directory")
@@ -270,19 +289,16 @@ def get_available_boards():
         from src.device_clone.board_config import list_supported_boards
 
         boards = list_supported_boards()
-        if not boards:
-            return [
-                "pcileech_35t325_x4",
-                "pcileech_75t484_x1",
-                "pcileech_100t484_x1",
-            ]
-        return sorted(boards)
+        return sorted(boards) if boards else get_fallback_boards()
     except Exception:
-        return [
-            "pcileech_35t325_x4",
-            "pcileech_75t484_x1",
-            "pcileech_100t484_x1",
-        ]
+        return get_fallback_boards()
+
+
+def get_fallback_boards():
+    """Get fallback board list when discovery fails."""
+    from src.device_clone.constants import BOARD_FALLBACKS
+
+    return list(BOARD_FALLBACKS)
 
 
 def check_sudo():
@@ -364,6 +380,9 @@ def rebuild_vfio_constants():
 
 def create_parser():
     """Create the main argument parser with subcommands."""
+    # Determine script name for auto-command detection
+    script_name = Path(sys.argv[0]).name
+
     parser = argparse.ArgumentParser(
         prog="pcileech",
         description="PCILeech Firmware Generator - Unified Entry Point",
@@ -373,7 +392,7 @@ Commands:
   build          Build firmware (CLI mode)
   tui            Launch interactive TUI
   flash          Flash firmware to device
-  check          Check VFIO configuration
+  check          Check VFIO configuration and ACS bypass requirements
   donor-template Generate donor info template
   version        Show version information
 
@@ -399,9 +418,7 @@ Environment Variables:
     )
 
     # Add global options
-    parser.add_argument(
-        "--version", action="version", version="PCILeech Firmware Generator v0.7.4"
-    )
+    parser.add_argument("--version", action="version", version=get_version())
     parser.add_argument(
         "--verbose", "-v", action="store_true", help="Enable verbose logging"
     )
@@ -414,6 +431,15 @@ Environment Variables:
         help="Skip automatic requirements checking",
     )
 
+    # Auto-detect command from script name for console scripts
+    auto_command = None
+    if script_name == "pcileech-build":
+        auto_command = "build"
+    elif script_name == "pcileech-tui":
+        auto_command = "tui"
+    elif script_name == "pcileech-generate":
+        auto_command = "build"  # generate is just an alias for build
+
     # Create subparsers for different modes
     subparsers = parser.add_subparsers(dest="command", help="Available commands")
 
@@ -436,6 +462,14 @@ Environment Variables:
     build_parser.add_argument(
         "--enable-variance", action="store_true", help="Enable manufacturing variance"
     )
+    build_parser.add_argument(
+        "--enable-error-injection",
+        action="store_true",
+        help=(
+            "Enable hardware error injection test hooks (AER). "
+            "Use only in controlled test environments."
+        ),
+    )
     build_parser.add_argument(
         "--build-dir", default="build", help="Directory for generated firmware files"
     )
@@ -449,10 +483,26 @@ Environment Variables:
     )
     build_parser.add_argument(
         "--device-type",
-        choices=["generic", "network", "storage", "audio", "graphics"],
+        choices=KNOWN_DEVICE_TYPES,
         default="generic",
         help="Override device type detection (default: auto-detect from class code)",
     )
+    build_parser.add_argument(
+        "--vivado-path",
+        help="Manual path to Vivado installation directory (e.g., /tools/Xilinx/2025.1/Vivado)",
+    )
+    build_parser.add_argument(
+        "--vivado-jobs",
+        type=int,
+        default=4,
+        help="Number of parallel jobs for Vivado builds (default: 4)",
+    )
+    build_parser.add_argument(
+        "--vivado-timeout",
+        type=int,
+        default=3600,
+        help="Timeout for Vivado operations in seconds (default: 3600)",
+    )
 
     # TUI command
     tui_parser = subparsers.add_parser("tui", help="Launch interactive TUI")
@@ -465,7 +515,9 @@ Environment Variables:
     flash_parser.add_argument("--device", help="USB device for flashing")
 
     # Check command (VFIO)
-    check_parser = subparsers.add_parser("check", help="Check VFIO configuration")
+    check_parser = subparsers.add_parser(
+        "check", help="Check VFIO configuration and ACS bypass requirements"
+    )
     check_parser.add_argument("--device", help="Specific device to check (BDF format)")
     check_parser.add_argument(
         "--interactive", "-i", action="store_true", help="Interactive remediation mode"
@@ -501,6 +553,10 @@ Environment Variables:
     )
     donor_parser.add_argument("--validate", help="Validate an existing donor info file")
 
+    # Set auto-detected command as default
+    if auto_command and not any(arg in sys.argv for arg in subparsers.choices):
+        parser.set_defaults(command=auto_command)
+
     return parser
 
 
@@ -527,6 +583,20 @@ def main():
 
     logger = get_logger(__name__)
 
+    # Handle console script auto-command detection
+    if not args.command:
+        script_name = Path(sys.argv[0]).name
+        if script_name == "pcileech-build":
+            args.command = "build"
+        elif script_name == "pcileech-tui":
+            args.command = "tui"
+        elif script_name == "pcileech-generate":
+            args.command = "build"
+        else:
+            # No command specified, show help
+            parser.print_help()
+            return 1
+
     # Check sudo requirements for hardware operations
     if args.command in ["build", "check"] and not check_sudo():
         log_error_safe(
@@ -587,6 +657,8 @@ def handle_build(args):
             cli_args.append("--advanced-sv")
         if args.enable_variance:
             cli_args.append("--enable-variance")
+        if getattr(args, "enable_error_injection", False):
+            cli_args.append("--enable-error-injection")
 
         if args.generate_donor_template:
             cli_args.extend(["--output-template", args.generate_donor_template])
@@ -626,9 +698,37 @@ def handle_tui(args):
             "Textual framework not installed. Install with: pip install textual rich psutil",
         )
 
-        # Import and run the TUI application
+        # Import TUI components
+        import platform
+
         from src.tui.main import PCILeechTUI
 
+        # Check OS compatibility first - PCILeech only supports Linux
+        current_os = platform.system()
+        is_compatible = current_os.lower() == "linux"
+
+        if not is_compatible:
+            log_error_safe(
+                logger,
+                "OS compatibility error: PCILeech requires Linux",
+                prefix="TUI",
+            )
+            log_error_safe(
+                logger,
+                f"Current platform: {current_os}. PCILeech TUI is only supported on Linux systems.",
+                prefix="TUI",
+            )
+            return 1
+
+        # Check sudo/root access (same as CLI mode)
+        if not check_sudo():
+            log_warning_safe(
+                logger,
+                "Continuing without root privileges - limited functionality.",
+                prefix="TUI",
+            )
+
+        # Launch the TUI application
         log_info_safe(logger, "Launching interactive TUI", prefix="TUI")
         app = PCILeechTUI()
         app.run()
@@ -802,7 +902,24 @@ def handle_check(args):
 def handle_version(args):
     """Handle version information."""
     logger = get_logger(__name__)
-    log_info_safe(logger, "PCILeech Firmware Generator v0.7.4", prefix="VERSION")
+
+    # Use centralized version resolver
+    try:
+        from src.utils.version_resolver import get_version_info
+
+        version_info = get_version_info()
+        version = version_info.get("version", "unknown")
+        title = version_info.get("title", "PCILeech Firmware Generator")
+        build_date = version_info.get("build_date", "unknown")
+        commit_hash = version_info.get("commit_hash", "unknown")
+
+        log_info_safe(logger, f"{title} v{version}", prefix="VERSION")
+        log_info_safe(logger, f"Build date: {build_date}", prefix="VERSION")
+        log_info_safe(logger, f"Commit hash: {commit_hash}", prefix="VERSION")
+
+    except ImportError:
+        log_info_safe(logger, get_version(), prefix="VERSION")
+
     log_info_safe(logger, "Copyright (c) 2024 PCILeech Project", prefix="VERSION")
     log_info_safe(logger, "Licensed under MIT License", prefix="VERSION")
 
