@@ -1,22 +1,6 @@
 #!/usr/bin/env python3
 """
 VFIO‑Assist - smarter VFIO diagnostics & auto‑fixer
-
-Usage (examples)
-----------------
-```bash
-# Full diagnostic with coloured TTY output
-sudo ./vfio_assist.py diagnose --device 0000:01:00.0
-
-# Attempt automatic remediation non‑interactively
-sudo ./vfio_assist.py fix --device 0000:01:00.0 --yes
-
-# Generate a remediation script only
-./vfio_assist.py script > vfio_fix.sh && chmod +x vfio_fix.sh
-
-# Machine‑readable JSON for a GitHub Action step
-./vfio_assist.py json --device 0000:01:00.0
-```
 """
 from __future__ import annotations
 
@@ -26,24 +10,20 @@ import logging
 import os
 import platform
 import re
+import shlex
 import shutil
 import subprocess
 import sys
 import textwrap
 from dataclasses import dataclass
-from enum import Enum
+from enum import Enum, auto
 from pathlib import Path
-from typing import List, Optional
+from typing import List, Optional, Tuple, Union
 
-# Add project root to path for utils imports
-sys.path.insert(0, str(Path(__file__).parent.parent.parent))
-from log_config import get_logger, setup_logging
-from src.string_utils import (
-    log_debug_safe,
-    log_error_safe,
-    log_info_safe,
-    log_warning_safe,
-)
+# Use consistent relative imports
+from ..log_config import get_logger, setup_logging
+from ..string_utils import (log_debug_safe, log_error_safe, log_info_safe,
+                            log_warning_safe, safe_format, safe_print_format)
 
 # ──────────────────────────────────────────────────────────────────────────────
 # Pretty terminal helpers
@@ -55,7 +35,7 @@ try:
     colorama_init()
 
     def colour(txt: str, col: str) -> str:  # noqa: D401 - short lambda‑style fn
-        return f"{col}{txt}{Style.RESET_ALL}"
+        return safe_format("{col}{txt}{reset}", col=col, txt=txt, reset=Style.RESET_ALL)
 
 except ImportError:  # colour optional - silently degrade
 
@@ -84,6 +64,115 @@ if not logging.getLogger().handlers:
 log = get_logger("vfio-assist")
 
 
+# ──────────────────────────────────────────────────────────────────────────────
+# Boot‑loader discovery + kernel‑arg helpers (drop‑in)
+# ──────────────────────────────────────────────────────────────────────────────
+class Boot(Enum):
+    GRUB2_LEGACY = auto()  # /boot/grub/grub.cfg
+    GRUB2_MODERN = auto()  # /boot/grub2/grub.cfg
+    GRUBBY = auto()  # grubby helper (RHEL/Fedora family)
+    SYSTEMD_BOOT = auto()  # /boot/loader/
+    UNKNOWN = auto()
+
+
+def _detect_boot() -> Boot:
+    if shutil.which("grubby"):
+        return Boot.GRUBBY
+    if Path("/boot/loader/entries").exists():
+        return Boot.SYSTEMD_BOOT
+    if Path("/boot/grub2/grub.cfg").exists():
+        return Boot.GRUB2_MODERN
+    if Path("/boot/grub/grub.cfg").exists():
+        return Boot.GRUB2_LEGACY
+    return Boot.UNKNOWN
+
+
+def _dedup(existing: str, new: Tuple[str, ...]) -> str:
+    parts = existing.strip('"').split()
+    for arg in new:
+        if arg not in parts:
+            parts.append(arg)
+    return '"' + " ".join(parts) + '"'
+
+
+def _sed_update(file: str, args: Tuple[str, ...]) -> str:
+    # gnu‑sed command that inserts args exactly once
+    return (
+        r"""sudo sed -i -E """
+        r"""'s/^(GRUB_CMDLINE_LINUX(_DEFAULT)?=)("(.*)")/\1"""
+        + _dedup(r"\3", args)
+        + r"/' "
+        "" + shlex.quote(file)
+    )
+
+
+def _cmds_for_args(args: Tuple[str, ...]) -> list[str]:
+    boot = _detect_boot()
+    joined = " ".join(args)
+
+    if boot == Boot.GRUBBY:
+        return [
+            safe_format(
+                "sudo grubby --update-kernel=ALL --args {args}",
+                args=shlex.quote(joined),
+            ),
+            "sudo reboot",
+        ]
+
+    if boot == Boot.GRUB2_MODERN:
+        return [
+            _sed_update("/etc/default/grub", args),
+            "sudo grub2-mkconfig -o /boot/grub2/grub.cfg",
+            "sudo reboot",
+        ]
+
+    if boot == Boot.GRUB2_LEGACY:
+        return [
+            _sed_update("/etc/default/grub", args),
+            "sudo grub-mkconfig -o /boot/grub/grub.cfg",
+            "sudo reboot",
+        ]
+
+    if boot == Boot.SYSTEMD_BOOT:
+        kcmd = Path("/etc/kernel/cmdline")
+        if kcmd.exists():
+            current = kcmd.read_text().strip().split()
+            add = [a for a in args if a not in current]
+            if add:
+                return [
+                    safe_format(
+                        "echo {args} | sudo tee -a /etc/kernel/cmdline",
+                        args=" ".join(add),
+                    ),
+                    "sudo kernelstub -A || sudo bootctl update",
+                    "sudo reboot",
+                ]
+            return [
+                "# kernel args already present – reboot if you just installed a new kernel"
+            ]
+        return ["# Could not locate /etc/kernel/cmdline – edit manually"]
+
+    return [
+        "# Unknown boot loader; append these once then reboot:",
+        safe_format("# {args}", args=joined),
+    ]
+
+
+def _kernel_param_commands() -> list[str]:
+    return _cmds_for_args(("intel_iommu=on", "amd_iommu=on", "iommu=pt"))
+
+
+def _kernel_param_commands_with_acs() -> list[str]:
+    return _cmds_for_args(
+        (
+            "intel_iommu=on",
+            "amd_iommu=on",
+            "iommu=pt",
+            "pcie_acs_override=downstream,multifunction",
+        )
+    )
+
+
 # ──────────────────────────────────────────────────────────────────────────────
 # Data structures
 # ──────────────────────────────────────────────────────────────────────────────
@@ -101,6 +190,7 @@ class Check:
     message: str
     remediation: Optional[str] = None
     commands: Optional[List[str]] = None
+    prefix: Optional[str] = None
 
 
 @dataclass
@@ -194,19 +284,21 @@ class Diagnostics:
                 prefix="VFIO",
                 error=str(e),
             )
-            log.error("Full exception details:", exc_info=True)
+            log_error_safe(
+                log, "Full exception details available in logs", prefix="VFIO"
+            )
             # Add an error check to indicate the diagnostic failure
             self._append(
                 name="Diagnostic Engine",
                 status=Status.ERROR,
-                message=f"Diagnostic engine failed: {e}",
+                message=safe_format("Diagnostic engine failed: {error}", error=str(e)),
                 remediation="Check system logs and retry",
             )
             return Report(Status.ERROR, self.checks, self.device_bdf, False)
 
     # Private helpers ---------------------------------------------------------
     @staticmethod
-    def _path_exists(path: str | Path) -> bool:
+    def _path_exists(path: Union[str, Path]) -> bool:
         return Path(path).exists()
 
     def _append(self, **kw):  # tiny helper
@@ -233,7 +325,9 @@ class Diagnostics:
             self._append(
                 name="Platform",
                 status=Status.ERROR,
-                message=f"Unsupported OS: {platform.system()}",
+                message=safe_format(
+                    "Unsupported OS: {os_name}", os_name=platform.system()
+                ),
                 remediation="Run on a Linux system with VFIO support",
             )
 
@@ -287,11 +381,13 @@ class Diagnostics:
             log_error_safe(
                 log, "Failed to check IOMMU hardware support: {error}", error=str(e)
             )
-            log.error("Full exception details:", exc_info=True)
+            log_error_safe(log, "Full exception details available in logs")
             self._append(
                 name="IOMMU HW",
                 status=Status.WARNING,
-                message=f"Could not parse /proc/cpuinfo: {e}",
+                message=safe_format(
+                    "Could not parse /proc/cpuinfo: {error}", error=str(e)
+                ),
             )
 
     def _check_kernel_params(self):
@@ -315,15 +411,44 @@ class Diagnostics:
             iommu_params = ["intel_iommu=on", "amd_iommu=on", "iommu=pt", "iommu=on"]
             found_params = [param for param in iommu_params if param in cmdline]
 
+            # Check for ACS bypass parameters
+            acs_params = [
+                "pcie_acs_override=downstream",
+                "pcie_acs_override=multifunction",
+                "pcie_acs_override=downstream,multifunction",
+            ]
+            found_acs = [param for param in acs_params if param in cmdline]
+
             log_debug_safe(log, "Found IOMMU parameters: {params}", params=found_params)
+            log_debug_safe(
+                log, "Found ACS override parameters: {params}", params=found_acs
+            )
 
             if found_params:
                 log_debug_safe(log, "IOMMU enabled in kernel cmdline")
+                message = safe_format(
+                    "IOMMU enabled in cmdline: {params}", params=", ".join(found_params)
+                )
+                if found_acs:
+                    # Check if ACS override is actually supported
+                    acs_supported = self._test_acs_override_support()
+                    if acs_supported:
+                        message += safe_format(
+                            ", ACS override: {acs_params}",
+                            acs_params=", ".join(found_acs),
+                        )
+                    else:
+                        message += ", ACS override params present but kernel doesn't support them"
+
                 self._append(
                     name="Kernel cmdline",
                     status=Status.OK,
-                    message=f"IOMMU enabled in cmdline: {', '.join(found_params)}",
+                    message=message,
                 )
+
+                # Check if ACS bypass might be needed
+                if not found_acs and self.device_bdf:
+                    self._check_acs_bypass_need()
             else:
                 log_warning_safe(log, "No IOMMU parameters found in kernel cmdline")
                 fix = "Enable intel_iommu=on and/or amd_iommu=on iommu=pt in grub"
@@ -332,46 +457,21 @@ class Diagnostics:
                     status=Status.ERROR,
                     message="IOMMU not present in kernel parameters",
                     remediation=fix,
-                    commands=self._kernel_param_commands(),
+                    commands=_kernel_param_commands(),
                 )
         except Exception as e:
             log_error_safe(
                 log, "Failed to check kernel parameters: {error}", error=str(e)
             )
-            log.error("Full exception details:", exc_info=True)
+            log_error_safe(log, "Full exception details available in logs")
             self._append(
                 name="Kernel cmdline",
                 status=Status.ERROR,
-                message=f"Failed to read /proc/cmdline: {e}",
+                message=safe_format(
+                    "Failed to read /proc/cmdline: {error}", error=str(e)
+                ),
             )
 
-    @staticmethod
-    def _kernel_param_commands() -> List[str]:
-        cmd: list[str] = []
-        if shutil.which("grubby"):
-            cmd.append(
-                'sudo grubby --update-kernel=ALL --args "intel_iommu=on amd_iommu=on iommu=pt"'
-            )
-        elif Path("/etc/default/grub").exists():
-            # Minimal fallback; user should still review
-            cmd.extend(
-                [
-                    'sudo sed -Ei \'s/GRUB_CMDLINE_LINUX=("[^"]*)/GRUB_CMDLINE_LINUX="\\1 intel_iommu=on amd_iommu=on iommu=pt"/\' /etc/default/grub',
-                    "sudo update-grub",
-                ]
-            )
-        elif Path(
-            "/etc/kernel/cmdline"
-        ).exists():  # systemd‑boot (Fedora Silverblue, etc.)
-            cmd.extend(
-                [
-                    "echo 'intel_iommu=on amd_iommu=on iommu=pt' | sudo tee -a /etc/kernel/cmdline",
-                    "sudo rpm-ostree initramfs --enable",
-                ]
-            )
-        cmd.append("sudo reboot # required for kernel param changes")
-        return cmd
-
     def _check_modules(self):
         """Check if required VFIO kernel modules are loaded."""
         log_debug_safe(log, "Checking VFIO kernel modules")
@@ -381,7 +481,7 @@ class Diagnostics:
         loaded = []
 
         for module in required:
-            module_path = f"/sys/module/{module}"
+            module_path = safe_format("/sys/module/{module}", module=module)
             if self._path_exists(module_path):
                 loaded.append(module)
                 log_debug_safe(log, "Module {module} is loaded", module=module)
@@ -401,7 +501,9 @@ class Diagnostics:
             self._append(
                 name="Kernel modules",
                 status=Status.OK,
-                message=f"All VFIO modules loaded: {', '.join(loaded)}",
+                message=safe_format(
+                    "All VFIO modules loaded: {modules}", modules=", ".join(loaded)
+                ),
             )
         else:
             severity = Status.ERROR if len(missing) == len(required) else Status.WARNING
@@ -414,9 +516,14 @@ class Diagnostics:
             self._append(
                 name="Kernel modules",
                 status=severity,
-                message="Missing modules: " + ", ".join(missing),
+                message=safe_format(
+                    "Missing modules: {modules}", modules=", ".join(missing)
+                ),
                 remediation="Load required modules with modprobe",
-                commands=[f"sudo modprobe {m.replace('_', '-')}" for m in missing],
+                commands=[
+                    safe_format("sudo modprobe {module}", module=m.replace("_", "-"))
+                    for m in missing
+                ],
             )
 
     def _check_vfio_driver_path(self):
@@ -436,25 +543,34 @@ class Diagnostics:
 
     # Device‑specific ---------------------------------------------------------
     def _device_exists(self):
-        device_path = Path(f"/sys/bus/pci/devices/{self.device_bdf}")
+        device_path = Path(
+            safe_format("/sys/bus/pci/devices/{bdf}", bdf=self.device_bdf)
+        )
         if device_path.exists():
             vendor = (device_path / "vendor").read_text().strip()
             device = (device_path / "device").read_text().strip()
             self._append(
                 name="Device",
                 status=Status.OK,
-                message=f"{self.device_bdf} ({vendor}:{device}) present",
+                message=safe_format(
+                    "{bdf} ({vendor}:{device}) present",
+                    bdf=self.device_bdf,
+                    vendor=vendor,
+                    device=device,
+                ),
             )
         else:
             self._append(
                 name="Device",
                 status=Status.ERROR,
-                message=f"PCI device {self.device_bdf} not found",
+                message=safe_format("PCI device {bdf} not found", bdf=self.device_bdf),
                 remediation="Check BDF with lspci ‑D",
             )
 
     def _device_iommu_group(self):
-        group_link = Path(f"/sys/bus/pci/devices/{self.device_bdf}/iommu_group")
+        group_link = Path(
+            safe_format("/sys/bus/pci/devices/{bdf}/iommu_group", bdf=self.device_bdf)
+        )
         log_debug_safe(
             log,
             "Checking IOMMU group link: {group_link}",
@@ -475,7 +591,9 @@ class Diagnostics:
                 )
 
                 # Check if the group directory exists
-                group_dir = Path(f"/sys/kernel/iommu_groups/{group}")
+                group_dir = Path(
+                    safe_format("/sys/kernel/iommu_groups/{group}", group=group)
+                )
                 if group_dir.exists():
                     # List devices in the group for debugging
                     devices_dir = group_dir / "devices"
@@ -500,7 +618,9 @@ class Diagnostics:
                             )
 
                 self._append(
-                    name="IOMMU group", status=Status.OK, message=f"Group {group}"
+                    name="IOMMU group",
+                    status=Status.OK,
+                    message=safe_format("Group {group}", group=group),
                 )
             except OSError as e:
                 log_debug_safe(
@@ -512,11 +632,15 @@ class Diagnostics:
                 self._append(
                     name="IOMMU group",
                     status=Status.ERROR,
-                    message=f"Failed to read IOMMU group symlink: {e}",
+                    message=safe_format(
+                        "Failed to read IOMMU group symlink: {error}", error=str(e)
+                    ),
                 )
         else:
             # Check if device exists at all
-            device_path = Path(f"/sys/bus/pci/devices/{self.device_bdf}")
+            device_path = Path(
+                safe_format("/sys/bus/pci/devices/{bdf}", bdf=self.device_bdf)
+            )
             if device_path.exists():
                 log_debug_safe(
                     log,
@@ -540,7 +664,9 @@ class Diagnostics:
                 self._append(
                     name="IOMMU group",
                     status=Status.ERROR,
-                    message=f"Device {self.device_bdf} not found in sysfs",
+                    message=safe_format(
+                        "Device {bdf} not found in sysfs", bdf=self.device_bdf
+                    ),
                     prefix="VFIO",
                 )
 
@@ -553,7 +679,9 @@ class Diagnostics:
             )
             return
 
-        link = Path(f"/sys/bus/pci/devices/{self.device_bdf}/driver")
+        link = Path(
+            safe_format("/sys/bus/pci/devices/{bdf}/driver", bdf=self.device_bdf)
+        )
         log_debug_safe(
             log,
             "Checking driver binding for {device} at {link}",
@@ -590,7 +718,7 @@ class Diagnostics:
                     self._append(
                         name="Driver",
                         status=Status.WARNING,
-                        message=f"Bound to {driver}",
+                        message=safe_format("Bound to {driver}", driver=driver),
                         remediation="Will need to rebind to vfio-pci",
                         commands=self._bind_commands(self.device_bdf, driver),
                     )
@@ -605,7 +733,9 @@ class Diagnostics:
                 self._append(
                     name="Driver",
                     status=Status.ERROR,
-                    message=f"Failed to read driver symlink: {e}",
+                    message=safe_format(
+                        "Failed to read driver symlink: {error}", error=str(e)
+                    ),
                 )
         else:
             log_debug_safe(
@@ -625,16 +755,23 @@ class Diagnostics:
     def _bind_commands(bdf: str, current: Optional[str]) -> List[str]:
         cmds: list[str] = [
             (
-                f"echo '{bdf}' | sudo tee /sys/bus/pci/devices/{bdf}/driver/unbind"
+                safe_format(
+                    "echo '{bdf}' | sudo tee /sys/bus/pci/devices/{bdf}/driver/unbind",
+                    bdf=bdf,
+                )
                 if current
                 else ""
             ),
-            f"echo '{bdf}' | sudo tee /sys/bus/pci/drivers/vfio-pci/bind",
+            safe_format(
+                "echo '{bdf}' | sudo tee /sys/bus/pci/drivers/vfio-pci/bind", bdf=bdf
+            ),
         ]
         return [c for c in cmds if c]
 
     def _device_node(self):
-        link = Path(f"/sys/bus/pci/devices/{self.device_bdf}/iommu_group")
+        link = Path(
+            safe_format("/sys/bus/pci/devices/{bdf}/iommu_group", bdf=self.device_bdf)
+        )
         log_debug_safe(
             log,
             "Checking VFIO device node for {device}",
@@ -662,7 +799,7 @@ class Diagnostics:
                 prefix="VFIO",
             )
 
-            node = Path(f"/dev/vfio/{group}")
+            node = Path(safe_format("/dev/vfio/{group}", group=group))
             log_debug_safe(log, "Checking VFIO node: {node}", node=node, prefix="VFIO")
 
             if node.exists():
@@ -717,7 +854,9 @@ class Diagnostics:
                 self._append(
                     name="/dev/vfio node",
                     status=Status.WARNING,
-                    message=f"{node} missing (will appear after binding)",
+                    message=safe_format(
+                        "{node} missing (will appear after binding)", node=str(node)
+                    ),
                 )
         except OSError as e:
             log_debug_safe(
@@ -729,10 +868,200 @@ class Diagnostics:
             self._append(
                 name="/dev/vfio node",
                 status=Status.ERROR,
-                message=f"Failed to determine VFIO node: {e}",
+                message=safe_format(
+                    "Failed to determine VFIO node: {error}", error=str(e)
+                ),
             )
 
-    # Overall --------------------------------------------------------------
+    def _check_acs_bypass_need(self):
+        """Check if ACS bypass might be needed for proper IOMMU group isolation."""
+        if not self.device_bdf:
+            return
+
+        log_debug_safe(
+            log,
+            "Checking if ACS bypass might be needed for {device}",
+            device=self.device_bdf,
+            prefix="VFIO",
+        )
+
+        try:
+            # Get the target device's IOMMU group
+            group_link = Path(f"/sys/bus/pci/devices/{self.device_bdf}/iommu_group")
+            if not group_link.exists():
+                return
+
+            group_target = os.readlink(group_link)
+            group = os.path.basename(group_target)
+
+            # Check how many devices are in this IOMMU group
+            group_devices_path = Path(
+                safe_format("/sys/kernel/iommu_groups/{group}/devices", group=group)
+            )
+            if not group_devices_path.exists():
+                return
+
+            devices = list(group_devices_path.iterdir())
+            device_count = len(devices)
+
+            log_debug_safe(
+                log,
+                "Device {device} is in IOMMU group {group} with {count} devices",
+                device=self.device_bdf,
+                group=group,
+                count=device_count,
+                prefix="VFIO",
+            )
+
+            if device_count > 1:
+                # Multiple devices in the same group - ACS bypass likely needed
+                device_names = [d.name for d in devices]
+
+                # Check if this looks like PCIe bridge isolation issue
+                bridges = [d for d in device_names if self._is_pci_bridge(d)]
+
+                other_devices = [d for d in device_names if d != self.device_bdf]
+                warning_msg = safe_format(
+                    "Device shares IOMMU group {group} with {count} other device(s): {devices}",
+                    group=group,
+                    count=device_count - 1,
+                    devices=", ".join(other_devices),
+                )
+
+                if bridges:
+                    warning_msg += safe_format(
+                        ". PCIe bridges detected: {bridges}", bridges=", ".join(bridges)
+                    )
+
+                # Check if ACS override is actually supported by testing the parameter
+                acs_supported = self._test_acs_override_support()
+
+                if acs_supported:
+                    remediation = "Enable ACS bypass with pcie_acs_override=downstream,multifunction"
+                    commands = _kernel_param_commands_with_acs()
+                else:
+                    remediation = "Device isolation requires ACS override patch (not available in this kernel)"
+                    commands = [
+                        "# ACS override is NOT supported in this kernel",
+                        "# General solutions:",
+                        "# 1. Check if your distribution provides ACS-patched kernels",
+                        "# 2. Build custom kernel with ACS patch",
+                        "# 3. Switch to distribution with ACS support (Proxmox, Arch AUR)",
+                        "# 4. Check BIOS for ACS/PCIe override options",
+                        "# 5. Consider different hardware topology",
+                    ]
+
+                self._append(
+                    name="ACS bypass",
+                    status=Status.WARNING,
+                    message=warning_msg,
+                    remediation=remediation,
+                    commands=commands,
+                )
+            else:
+                log_debug_safe(
+                    log,
+                    "Device {device} is isolated in its own IOMMU group - ACS bypass not needed",
+                    device=self.device_bdf,
+                    prefix="VFIO",
+                )
+                self._append(
+                    name="ACS bypass",
+                    status=Status.OK,
+                    message=safe_format(
+                        "Device isolated in IOMMU group {group}", group=group
+                    ),
+                )
+
+        except Exception as e:
+            log_debug_safe(
+                log,
+                "Failed to check ACS bypass need: {error}",
+                error=str(e),
+                prefix="VFIO",
+            )
+            self._append(
+                name="ACS bypass",
+                status=Status.WARNING,
+                message=safe_format(
+                    "Could not determine if ACS bypass is needed: {error}", error=str(e)
+                ),
+            )
+
+    def _is_pci_bridge(self, bdf: str) -> bool:
+        """Check if a device is a PCIe bridge by examining its class code."""
+        try:
+            class_path = Path(safe_format("/sys/bus/pci/devices/{bdf}/class", bdf=bdf))
+            if class_path.exists():
+                class_code = class_path.read_text().strip()
+                # PCIe bridges typically have class code 0x060400 or 0x060401
+                return class_code.startswith("0x0604")
+        except Exception:
+            pass
+        return False
+
+    def _get_kernel_version(self) -> Optional[str]:
+        """Get the current kernel version string."""
+        try:
+            result = subprocess.run(
+                ["uname", "-r"], capture_output=True, text=True, timeout=5
+            )
+            if result.returncode == 0:
+                return result.stdout.strip()
+        except Exception:
+            pass
+        return None
+
+    def _test_acs_override_support(self) -> bool:
+        """Test if the current kernel supports ACS override by checking if the parameter exists."""
+        # 1. live sysfs parameter
+        if Path("/sys/module/pci/parameters/pcie_acs_override").exists():
+            return True
+
+        # 2. kernel build config
+        kcfg = Path(safe_format("/boot/config-{release}", release=os.uname().release))
+        try:
+            if kcfg.exists():
+                for line in kcfg.read_text().splitlines():
+                    if line.startswith("CONFIG_PCI_QUIRKS="):
+                        return line.rstrip().endswith("y")
+        except PermissionError:
+            pass
+
+        # 3. modular pci driver parameters
+        try:
+            out = subprocess.run(
+                ["modinfo", "-p", "pci"],
+                capture_output=True,
+                text=True,
+                timeout=5,
+            )
+            if out.returncode == 0:
+                return "pcie_acs_override" in out.stdout
+        except (subprocess.SubprocessError, FileNotFoundError):
+            pass
+
+        return False
+
+    def _is_ubuntu(self) -> bool:
+        """Check if we're running on Ubuntu."""
+        try:
+            # Check /etc/os-release
+            os_release = Path("/etc/os-release")
+            if os_release.exists():
+                content = os_release.read_text()
+                return "ID=ubuntu" in content.lower() or "ubuntu" in content.lower()
+
+            # Fallback: check /etc/lsb-release
+            lsb_release = Path("/etc/lsb-release")
+            if lsb_release.exists():
+                content = lsb_release.read_text()
+                return "ubuntu" in content.lower()
+
+        except Exception:
+            pass
+        return False
+
     def _overall(self) -> Status:
         if any(c.status == Status.ERROR for c in self.checks):
             return Status.ERROR
@@ -775,15 +1104,32 @@ SYMBOLS = {
 
 def render(report: Report):
     print(colour("\n=== VFIO DIAGNOSTIC REPORT ===", Fore.CYAN))
-    print(f"Overall: {SYMBOLS[report.overall]} {report.overall.value.upper()}")
+    print(
+        safe_format(
+            "Overall: {symbol} {status}",
+            symbol=SYMBOLS[report.overall],
+            status=report.overall.value.upper(),
+        )
+    )
     if report.device_bdf:
-        print(f"Device : {report.device_bdf}")
-    print(f"Proceed: {'yes' if report.can_proceed else 'no'}\n")
+        print(safe_format("Device : {device}", device=report.device_bdf))
+    print(
+        safe_format(
+            "Proceed: {proceed}\n", proceed="yes" if report.can_proceed else "no"
+        )
+    )
     for ck in report.checks:
         sym = SYMBOLS.get(ck.status, "?")
-        print(f"{sym} {ck.name}: {ck.message}")
+        print(
+            safe_format(
+                "{symbol} {name}: {message}",
+                symbol=sym,
+                name=ck.name,
+                message=ck.message,
+            )
+        )
         if ck.remediation:
-            print("   · " + ck.remediation)
+            print(safe_format("   · {remediation}", remediation=ck.remediation))
     print()
 
 
@@ -868,7 +1214,12 @@ def main(argv: list[str] | None = None):
         temp.write_text(script_text)
         temp.chmod(0o755)
         render(report)
-        print(colour(f"Remediation script written to {temp}", Fore.CYAN))
+        print(
+            colour(
+                safe_format("Remediation script written to {path}", path=str(temp)),
+                Fore.CYAN,
+            )
+        )
 
         if not args.yes:
             confirm = input("Run remediation script now? [y/N]: ").strip().lower()
