@@ -14,8 +14,9 @@ from datetime import datetime
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple
 
-from ..exceptions import DeviceConfigError, ValidationError
-from ..string_utils import safe_format, safe_log_format
+
+from src.exceptions import DeviceConfigError, ValidationError
+from src.string_utils import safe_format, safe_log_format, log_error_safe, log_info_safe
 
 logger = logging.getLogger(__name__)
 
@@ -31,20 +32,23 @@ class DonorInfoTemplateGenerator:
         Returns:
             Dict containing the complete template structure with blank/null values
         """
+        from src.utils.metadata import build_generation_metadata
+
         safe_log_format(
             logger, logging.INFO, "Generating blank donor info template", prefix="DONOR"
         )
 
+        # Use centralized metadata generation
+        metadata = build_generation_metadata(
+            device_bdf="",  # User to fill: e.g., "0000:03:00.0"
+            behavioral_data_included=True,
+            profile_capture_duration=None,  # User to fill: duration in seconds
+            comments="",  # User can add notes about the device
+            kernel_version="",  # User to fill: e.g., "6.1.0-15-amd64"
+        )
+
         template = {
-            "metadata": {
-                "generated_at": datetime.utcnow().isoformat() + "Z",
-                "device_bdf": "",  # User to fill: e.g., "0000:03:00.0"
-                "kernel_version": "",  # User to fill: e.g., "6.1.0-15-amd64"
-                "generator_version": "enhanced-v2.0",
-                "behavioral_data_included": True,
-                "profile_capture_duration": None,  # User to fill: duration in seconds
-                "comments": "",  # User can add notes about the device
-            },
+            "metadata": metadata,
             "device_info": {
                 "identification": {
                     "vendor_id": None,  # User to fill: e.g., 0x8086
@@ -312,6 +316,8 @@ class DonorInfoTemplateGenerator:
         Returns:
             Dict containing the minimal template structure
         """
+        from src.utils.metadata import build_generation_metadata
+
         safe_log_format(
             logger,
             logging.INFO,
@@ -319,13 +325,14 @@ class DonorInfoTemplateGenerator:
             prefix="DONOR",
         )
 
+        # Use centralized metadata generation
+        metadata = build_generation_metadata(
+            device_bdf="",  # User to fill: e.g., "0000:03:00.0"
+            template_type="minimal",
+        )
+
         template = {
-            "metadata": {
-                "generated_at": datetime.utcnow().isoformat() + "Z",
-                "device_bdf": "",  # User to fill: e.g., "0000:03:00.0"
-                "generator_version": "minimal-v1.0",
-                "template_type": "minimal",
-            },
+            "metadata": metadata,
             "device_info": {
                 "identification": {
                     "vendor_id": None,  # User to fill: e.g., 0x8086
@@ -382,7 +389,7 @@ class DonorInfoTemplateGenerator:
             )
         except Exception as e:
             error_msg = safe_format("Failed to save template: {error}", error=str(e))
-            logger.error(error_msg)
+            log_error_safe(logger, error_msg, prefix="DONOR")
             raise DeviceConfigError(error_msg) from e
 
     @staticmethod
@@ -413,7 +420,7 @@ class DonorInfoTemplateGenerator:
             )
         except Exception as e:
             error_msg = safe_format("Failed to save template: {error}", error=str(e))
-            logger.error(error_msg)
+            log_error_safe(logger, error_msg, prefix="DONOR")
             raise DeviceConfigError(error_msg) from e
 
     @staticmethod
@@ -424,23 +431,97 @@ class DonorInfoTemplateGenerator:
         Returns:
             String containing the template with comments
         """
-        # This returns a string with comments that wouldn't be valid JSON
-        # but is useful for documentation purposes
-        template_with_comments = """
-{
-  "metadata": {
-    "generated_at": "2025-01-19T12:30:45Z",  // Auto-generated timestamp
-    "device_bdf": "",  // PCIe Bus:Device.Function (e.g., "0000:03:00.0")
-    "kernel_version": "",  // Kernel version used for profiling
-    "generator_version": "0.7.3",
-    "behavioral_data_included": true,
-    "profile_capture_duration": null,  // Duration of behavioral capture in seconds
-    "comments": ""  // Any additional notes about the device
-  },
-  // ... rest of template with comments ...
-}
-"""
-        return template_with_comments
+        # Build from the blank template to avoid drift and emit valid JSON
+        base = DonorInfoTemplateGenerator.generate_blank_template()
+
+        # Attach $comment fields to key sections. Keep it minimal to satisfy
+        # documentation needs without duplicating schema.
+        base["$comment"] = (
+            "Template with explanatory comments provided via $comment fields. "
+            "Values are blanks for users to fill; unknown keys may be ignored by tooling."
+        )
+
+        # Metadata comments
+        md = base.get("metadata", {})
+        if isinstance(md, dict):
+            md["$comment"] = (
+                "Generation metadata; device_bdf is PCIe Bus:Device.Function "
+                "(e.g., 0000:03:00.0). generated_at and generator_version are auto-populated."
+            )
+
+        # Device info section comment
+        di = base.get("device_info", {})
+        if isinstance(di, dict):
+            di["$comment"] = (
+                "Static device description: identification (VID/DID/etc), "
+                "capabilities (PCIe features), and BAR layout."
+            )
+            ident = di.get("identification", {})
+            if isinstance(ident, dict):
+                ident["$comment"] = (
+                    "PCI identification fields. vendor_id/device_id required; "
+                    "class_code is the PCI class (e.g., 0x020000)."
+                )
+            caps = di.get("capabilities", {})
+            if isinstance(caps, dict):
+                caps["$comment"] = (
+                    "Key PCIe capabilities and negotiated link parameters."
+                )
+            bars = di.get("bars", [])
+            if isinstance(bars, list) and bars:
+                # Comment the first BAR object shape
+                if isinstance(bars[0], dict):
+                    bars[0]["$comment"] = (
+                        "Describe each BAR: type=memory|io, size in bytes, "
+                        "prefetchable true/false, 64bit true/false, and purpose."
+                    )
+
+        # Behavioral profile section comment
+        bp = base.get("behavioral_profile", {})
+        if isinstance(bp, dict):
+            bp["$comment"] = (
+                "Behavioral profiling hints: init timing/sequence, runtime patterns, "
+                "DMA behavior, error responses, and performance."
+            )
+            init = bp.get("initialization", {})
+            if isinstance(init, dict):
+                init["$comment"] = (
+                    "Initialization/reset characteristics and any firmware load steps."
+                )
+            runtime = bp.get("runtime_behavior", {})
+            if isinstance(runtime, dict):
+                runtime["$comment"] = (
+                    "Interrupt and memory access patterns plus timing characteristics."
+                )
+            dma = bp.get("dma_behavior", {})
+            if isinstance(dma, dict):
+                dma["$comment"] = (
+                    "DMA engines, directions, alignment, and timing patterns."
+                )
+
+        # Advanced features
+        adv = base.get("advanced_features", {})
+        if isinstance(adv, dict):
+            adv["$comment"] = (
+                "Optional/advanced capabilities: vendor protocols, security, virtualization, debug."
+            )
+
+        # Emulation hints
+        eh = base.get("emulation_hints", {})
+        if isinstance(eh, dict):
+            eh["$comment"] = (
+                "Prioritized features, quirks, and optimization/testing recommendations."
+            )
+
+        # Extended behavioral data
+        ext = base.get("extended_behavioral_data", {})
+        if isinstance(ext, dict):
+            ext["$comment"] = (
+                "Deeper behavioral datasets: workload profiles, transitions, scaling, compatibility."
+            )
+
+        # Serialize as pretty JSON
+        return json.dumps(base, indent=2, sort_keys=False)
 
     def __init__(self):
         """Initialize the template generator."""
@@ -590,7 +671,7 @@ class DonorInfoTemplateGenerator:
             error_msg = safe_format(
                 "Failed to run lspci for device {bdf}: {error}", bdf=bdf, error=str(e)
             )
-            logger.error(error_msg)
+            log_error_safe(logger, error_msg, prefix="DONOR")
             raise DeviceConfigError(error_msg) from e
         except Exception as e:
             error_msg = safe_format(
@@ -598,7 +679,7 @@ class DonorInfoTemplateGenerator:
                 bdf=bdf,
                 error=str(e),
             )
-            logger.error(error_msg)
+            log_error_safe(logger, error_msg, prefix="DONOR")
             raise DeviceConfigError(error_msg) from e
 
         return template
@@ -704,13 +785,13 @@ class DonorInfoTemplateGenerator:
 
         except FileNotFoundError:
             error_msg = safe_format("Template file not found: {path}", path=filepath)
-            logger.error(error_msg)
+            log_error_safe(logger, error_msg, prefix="DONOR")
             raise ValidationError(error_msg)
         except json.JSONDecodeError as e:
             error_msg = safe_format(
                 "Invalid JSON in {path}: {error}", path=filepath, error=str(e)
             )
-            logger.error(error_msg)
+            log_error_safe(logger, error_msg, prefix="DONOR")
             raise ValidationError(error_msg) from e
 
     @staticmethod
