@@ -30,7 +30,7 @@ class BoardDiscovery:
     # Known board directory patterns in pcileech-fpga
     BOARD_DIR_PATTERNS = {
         # Legacy boards
-        "PCIeSquirrel": {"name": "35t", "fpga_part": "xc7a35tcsg324-2"},
+        "PCIeSquirrel": {"name": "35t", "fpga_part": "xc7a35tfgg484-2"},
         "PCIeEnigmaX1": {"name": "75t", "fpga_part": "xc7a75tfgg484-2"},
         "XilinxZDMA": {"name": "100t", "fpga_part": "xczu3eg-sbva484-1-e"},
         # Modern boards with more specific patterns
@@ -47,7 +47,12 @@ class BoardDiscovery:
         "35t484_x1": {"fpga_part": "xc7a35tfgg484-2", "max_lanes": 1},
         "35t325_x4": {"fpga_part": "xc7a35tcsg324-2", "max_lanes": 4},
         "35t325_x1": {"fpga_part": "xc7a35tcsg324-2", "max_lanes": 1},
-        "100t484-1": {"fpga_part": "xczu3eg-sbva484-1-e", "max_lanes": 1},
+        "100t484-1": {
+            # CaptainDMA 100T board uses Artix-7 100T in FGG484 package
+            "fpga_part": "xc7a100tfgg484-1",
+            "max_lanes": 1,
+            "canonical_name": "pcileech_100t484_x1",
+        },
     }
 
     @classmethod
@@ -85,7 +90,10 @@ class BoardDiscovery:
             for subdir, config in cls.CAPTAINDMA_BOARDS.items():
                 board_path = captaindma_root / subdir
                 if board_path.exists() and board_path.is_dir():
-                    board_name = f"pcileech_{subdir.replace('-', '_')}"
+                    # Use canonical name if specified, otherwise construct standard name
+                    board_name = config.get(
+                        "canonical_name", f"pcileech_{subdir.replace('-', '_')}"
+                    )
                     boards[board_name] = cls._analyze_board(
                         board_path, {"name": board_name, **config}
                     )
