@@ -5,8 +5,8 @@ This package provides a modern Text User Interface (TUI) for the PCILeech
 firmware generation workflow, built with Textual framework.
 """
 
-__version__ = "1.0.0"
-__author__ = "PCILeech Development Team"
+__version__ = "0.5.0"
+__author__ = "Ramsey McGrath"
 
 from .main import PCILeechTUI
 
