@@ -5,9 +5,16 @@ This module contains the core service classes that handle business logic
 and integration with the existing PCILeech functionality.
 """
 
+from .background_monitor import BackgroundMonitor
 from .build_orchestrator import BuildOrchestrator
 from .config_manager import ConfigManager
 from .device_manager import DeviceManager
 from .status_monitor import StatusMonitor
 
-__all__ = ["DeviceManager", "ConfigManager", "BuildOrchestrator", "StatusMonitor"]
+__all__ = [
+    "BackgroundMonitor",
+    "DeviceManager",
+    "ConfigManager",
+    "BuildOrchestrator",
+    "StatusMonitor",
+]
